/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Bo Yang<techrazy.yang@gmail.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

 //Global object to store the broswer in which the ms builder is opened
var msOpenTab = null;
var msOpen = false;
const FIELD_MICSUM_GEN_URI    = "microsummary/generatorURI";

//functions used to watch tabs
function tabChanged(event){
	var tb = document.getElementById("microsummary-toolbar");

	if (msOpenTab==gBrowser.selectedTab.linkedBrowser&&msOpen){
		tb.collapsed = false;
		advancedMSBuilder.attachEventHandlers();
	}else{
		tb.collapsed = true;
		advancedMSBuilder.detachEventHandlers();
	}
}

function tabClosed(event){
	var tb = document.getElementById("microsummary-toolbar");
	if (msOpenTab==event.target.linkedBrowser){
		advancedMSBuilder.closeMSEditor();
	}
}


//function used to ignore some text node in XML parsing
function findNextActualNode( element ){
	while (element&&element.nodeType==element.TEXT_NODE){
		element = element.nextSibling;
	}
	return element;
}

function iterateWindows(win, handler)
{
    if (!win || !win.document)
        return;

    handler(win);
    
    for (var i = 0; i < win.frames.length; ++i)
    {
        var subWin = win.frames[i];
        if (subWin != win)
            iterateWindows(subWin, handler);
    }
}

var advancedMSBuilderListener =
{
  	QueryInterface: function(aIID)
  	{
   		if (aIID.equals(Ci.nsIWebProgressListener) ||
       		aIID.equals(Ci.nsISupportsWeakReference) ||
       		aIID.equals(Ci.nsISupports))
     			return this;
   		throw Components.results.NS_NOINTERFACE;
  	},
	
	onStateChange: function(aProgress, aRequest, aFlag, aStatus)
	{
	 if(aFlag & Ci.nsIWebProgressListener.STATE_STOP)
	 {
	 	if (aProgress.isLoadingDocument)
	 		advancedMSBuilder.reset();
	 }
	 return 0;
	},
	
	onLocationChange: function(aProgress, aRequest, aURI)
	{
	 	advancedMSBuilder.clear();
	 	return 0;
	},
	
	onProgressChange: function() {return 0;},
	onStatusChange: function() {return 0;},
	onSecurityChange: function() {return 0;},
	onLinkIconAvailable: function() {return 0;}
}

var msSelectionListener = {
	QueryInterface: function(aIID)
  	{
   		if (aIID.equals(Ci.nsISelectionListener) ||
       		aIID.equals(Ci.nsISupportsWeakReference) ||
       		aIID.equals(Ci.nsISupports))
     			return this;
   		throw Components.results.NS_NOINTERFACE;
  	},
	
	notifySelectionChanged: function( doc, sel, reason){
		if (advancedMSBuilder.editorCallback == null || sel.rangeCount != 1)
			return;

		var node = sel.focusNode;
		var desiredNode = null;
		var findOne = false;
		var onlyOne = false;
		var next = {
			dir: true,
			get: function( node ){
				if (this.dir)
					return node.nextSibling;
				else
					return node.previousSibling;
			}
		};
		if (sel.containsNode( node.previousSibling, true))
			next.dir = false;
		else if (sel.containsNode( node.nextSibling, true))
			next.dir = true;
		else
			onlyOne = true;

		for ( ; onlyOne == false && node != sel.anchorNode && node != null; node = next.get(node)){
			if (node.nodeName == "SPAN"){
				if (findOne == true){
					advancedMSBuilder.editorCallback(null);
					return;
				}
				findOne = true;
				desiredNode = node;
			}
		}
		//For the last node
		if (node!=null && node.nodeName == "SPAN"){
			alert(node + "and the value is :" + node.nodeValue);
			if (findOne == true)
				return;
			findOne = true;
			desiredNode = node;
		}

		if (desiredNode == null){
			advancedMSBuilder.editorCallback(null);
			return;
		}
		if (desiredNode.firstChild.nodeValue.replace(/ /g,"") == sel.toString().replace(/ /g,"")) 
			advancedMSBuilder.editorCallback(desiredNode);
		else
			advancedMSBuilder.editorCallback(null);
	}
}

var msStyleOptions = {
	outline: "lime",
	label: null,
	name: null,
	node: null,
	common: true,
	
	commonMouseUp: function( builder ) {
		var editor = document.getElementById("advanedMSEditor");
		var expression = "string(" + getXPath( builder.context.element ) + ")" ;
		var content = "normalize-space(" + expression + ")";
		var en = editor.createEntity( gBrowser.contentDocument, expression);
		if (en==null){
			return;
			//We need some helpful tooltip here to let the user know why he failed.
		}
		en.setAttribute("newvalue",content);
		if( en != null )
			editor.addEntity( en );
	},

	selectFromPage: function (builder) {
		var expression = "string(" + getXPath( builder.context.element ) + ")" ;
		var content = "normalize-space(" + expression + ")";
		var rv = advanceOptions.evaluate(content);
		if (rv==null)
			return;

		if (this.label!=null)
			this.label.value = '"' + rv + '"';
		this.node.setAttribute(this.name,content);
		this.outline = "lime";
		this.common = true;

		advanceOptions.refreshNode();
	},

	reset: function (){
		this.outline = "lime";
		this.common = true;
		this.label = null;
		this.name = null;
		this.node = null;
	},
	
	handleMouseUp: function( builder ){
		if (advancedMSBuilder.changeFrame(builder.context.element))
			return;
		
		if (this.common==true)
			this.commonMouseUp(builder);
		else
			this.selectFromPage(builder);
	}
}

var advancedMSBuilder = {
	uuid: null,
	_mseditor: null,
	get mseditor(){
		if(!this._mseditor)
			this._mseditor = document.getElementById("advanedMSEditor");
		return this._mseditor;
	},
	
  	_bms: null,
  	get bms() {
    		if(!this._bms)
      			this._bms = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].
                  		getService(Ci.nsINavBookmarksService);
    		return this._bms;
  	},

	_ans: null,
  	get ans() {
    		if(!this._ans)
      			this._ans = Cc["@mozilla.org/browser/annotation-service;1"].
                  		getService(Ci.nsIAnnotationService);
    		return this._ans;
  	},

  	_mss: null,
  	get mss() {
    		if(!this._mss)
      			this._mss = Cc["@mozilla.org/microsummary/service;1"].
                  		getService(Ci.nsIMicrosummaryService);
    		return this._mss;
  	},

	_dp: null,
	get dp() { 
		if(!this._dp)
			this._dp = Cc["@mozilla.org/xmlextras/domparser;1"].
								 createInstance(Ci.nsIDOMParser);
		return this._dp;
	},
	
	_ug: null,
	get ug() {
		if(!this._ug)
			this._ug = Cc["@mozilla.org/uuid-generator;1"].
								 getService(Ci.nsIUUIDGenerator);	
		return this._ug;
	},

	get generatorstring() {
		return this.buildTemplateString();
	},

	set generatorstring(value) {
		this.mseditor.content.body.innerHTML = "";
		var eles = this.parseFromString(this.mseditor.content,value);
		for (var i=0; i<eles.length; i++){
			if (eles[i].nodeType==eles[i].ELEMENT_NODE){
				this.mseditor.addEntity(eles[i]);
				advanceOptions.span = eles[i];
				advanceOptions.refreshNode();
				continue;
			}
			this.mseditor.content.body.appendChild(eles[i]);
			this.mseditor.editor.endOfDocument();
		}
	},

	parseFromString: function( doc, value ){
		var ret = [];
		var dom = this.dp.parseFromString( value, "text/xml" );

		//Using the ugly method to find the translate element
		var translate = dom.firstChild;
		for (var i=0; i<3; i++){
			translate = translate.firstChild;
			if (translate.nodeType==translate.TEXT_NODE)
				translate = findNextActualNode(translate);
		}
		translate = translate.nextSibling;
		translate = findNextActualNode(translate);

		var len = translate.childNodes.length;
		for (var i=0; i<len; i ++){
			var element = translate.childNodes[i];
			if (element.nodeType==element.TEXT_NODE)
				continue;
			switch (element.nodeName){
				case "text":
					var ele = doc.createTextNode(element.textContent);
					ret.push(ele);
					break;
				case "choose":
					var ele = this.parsePredicatorString(element);
					ret.push(ele);
					break;
				case "value-of":
					var exp = element.getAttributeNS("http://advmicrosummarygenerator.mozdev.org/custom","exp");
					var ele = this.mseditor.createEntity(content.document,exp);
					if (ele==null)
						break;
					var type = element.getAttributeNS("http://advmicrosummarygenerator.mozdev.org/custom","type");
					exp = element.getAttribute("select");
					ele.setAttribute("type",type);
					ele.setAttribute(type,exp);
					ret.push(ele);
					break;
				default:
					dump("There are somthing wrong in the String");
			}
		}
		return ret;
	},

	parsePredicatorString: function (element) {
		var exp = element.getAttributeNS("http://advmicrosummarygenerator.mozdev.org/custom","exp");
		var when = element.firstChild;
		when = findNextActualNode(when);

		var type = "predicator";
		var predicatortype = element.getAttributeNS("http://advmicrosummarygenerator.mozdev.org/custom","predicator");
		var ele = this.mseditor.createEntity(content.document,exp);
		ele.setAttribute("type",type);
		ele.setAttribute("predicatortype",predicatortype);

		ele.setAttribute("condition",when.getAttribute("test"));
		ele.setAttribute("expression",exp);
		var str = "if";
		while (when){
			var col = when.firstChild;
			col = findNextActualNode(col);
			switch (col.nodeName){
				case "value-of":
					var action = col.getAttributeNS("http://advmicrosummarygenerator.mozdev.org/custom","actiontype");
					ele.setAttribute(predicatortype+"-" + str,action);
					ele.setAttribute(predicatortype+"-" + str + "-" + action, col.getAttribute("select"));
					break;
				case "text":
					var action = "defineBySelf";
					ele.setAttribute(predicatortype+"-" + str,action);
					ele.setAttribute(predicatortype+"-" + str + "-" + action, col.textContent);
					break;
				default:
					dump("Error of explain the node");
			}
			when = when.nextSibling;
			when = findNextActualNode(when);
			str = "else";
		}
		return ele;
	},

	openMSEditor: function(){
		var command = document.getElementById("toogleCommand");
		command.setAttribute( "disabled",true);
		var bd = document.getElementById("toogleCommandFromContext");
		bd.setAttribute("disabled",true);
		
		var tb = document.getElementById("microsummary-toolbar");
		tb.collapsed = false ;
		
		gBrowser.selectedBrowser.addProgressListener( advancedMSBuilderListener, 
														Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
		msOpenTab = gBrowser.selectedTab.linkedBrowser;
		this.reset();
		msOpen = true;
	},

	openEditorFromContext: function() {
		this.openMSEditor();
		var ele = gContextMenu.target;
		var editor = document.getElementById("advanedMSEditor");
		var expression = "string(" + getXPath( ele ) + ")" ;
		var en = editor.createEntity( gBrowser.contentDocument, expression);
		if( en != null )
			editor.addEntity( en );
	},
	
	closeMSEditor: function(){
		var command = document.getElementById("toogleCommand")
		command.setAttribute( "disabled",false)
		var bd = document.getElementById("toogleCommandFromContext");
		bd.setAttribute("disabled",false);
		
		var tb = document.getElementById("microsummary-toolbar")
		tb.collapsed = true
		
		this.detachEventHandlers();
		
		gBrowser.selectedBrowser.removeProgressListener( advancedMSBuilderListener, 
														Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
		msOpen = false;
	},

	onPopup: function() {
		if (document.getElementById("microsummary-toolbar").getAttribute("collapsed")==false){
			return;
		}

		document.getElementById("toogleCommand").setAttribute("label",advanceOptions.strings.getString("normalTitleItem"));
		document.getElementById("toogleCommandFromContext").setAttribute("label",advanceOptions.strings.getString("normalTitleItem"));			
		
		var ios = Cc["@mozilla.org/network/io-service;1"]
                    			.getService(Ci.nsIIOService);
		var uri = ios.newURI(content.document.location,null,null);
		var ms = advancedMSBuilder.mss.getMicrosummaries(uri , -1);
		var it = ms.QueryInterface(Ci.nsIMicrosummarySet).Enumerate();
		while (it.hasMoreElements()){
			var m = it.getNext();
			m = m.QueryInterface(Ci.nsIMicrosummary);
			var url = m.generator.uri.spec;
			if (url.search(/urn:\{[0-9a-fA-F-]+\}/) == -1)
				continue;

			if (advancedMSBuilder.bookmarked( uri, m )){
				document.getElementById("bookmark").style.display = "none";
				document.getElementById("save").style.display = "block";
			}
			document.getElementById("toogleCommand").setAttribute("label",advanceOptions.strings.getString("editTitleItem"));
			document.getElementById("toogleCommandFromContext").setAttribute("label",advanceOptions.strings.getString("editTitleItem"));
		}
	},

	initPopup: function() {
		var context = document.getElementById("contentAreaContextMenu");
		var bookmark = document.getElementById("bookmarksMenuPopup");

		context.addEventListener("popupshowing",this.onPopup,false);
		bookmark.addEventListener("popupshowing",this.onPopup,false);
	},

	destroyPopup: function() {
		var context = document.getElementById("contentAreaContextMenu");
		var bookmark = document.getElementById("bookmarksMenuPopup");

		context.removeEventListener("popupshowing",this.onPopup,false);
		bookmark.removeEventListener("popupshowing",this.onPopup,false);
	},
		
	editorCallback: function(ele){
		advanceOptions.span = ele;
		advanceOptions.open();
	},
		
	mouseOver: function(evt){ advancedMSBuilder.handler.mouseOver(evt); evt.stopPropagation();},
	mouseOut:  function(evt){ advancedMSBuilder.handler.mouseOut(evt); evt.stopPropagation();},
	mouseDown: function(evt){ advancedMSBuilder.handler.mouseDown(evt); },
	mouseUp:   function(evt){ advancedMSBuilder.handler.mouseUp(evt); },
	mouseScroll: function(evt){ advancedMSBuilder.handler.mouseScroll(evt); evt.preventDefault(); evt.stopPropagation();},
	mouseClick: function(evt){ advancedMSBuilder.handler.mouseClick(evt); },
	keyPress:  function(evt){ advancedMSBuilder.handler.keyPress(evt); evt.preventDefault(); evt.stopPropagation();},
	
	attachEventHandlers: function(){
		var hookup = function(win){
			win.addEventListener("mouseover",advancedMSBuilder.mouseOver,true);
			win.addEventListener("mouseout",advancedMSBuilder.mouseOut,true);
			win.addEventListener("mousedown",advancedMSBuilder.mouseDown,true);
			win.addEventListener("mouseup",advancedMSBuilder.mouseUp,true);
			win.addEventListener("DOMMouseScroll",advancedMSBuilder.mouseScroll,true);
			win.addEventListener("click",advancedMSBuilder.mouseClick,true);
			win.addEventListener("keypress",advancedMSBuilder.keyPress,true);
		};
		iterateWindows(content.window, hookup);
	  
	  //absolutely, the code here need some modification to make the drag-and-drop area narrow
		var tb = document.getElementById("content");
		tb.addEventListener("draggesture",this.handler.dragGesture,true);
	},

	detachEventHandlers: function(){
		var unhookup = function(win){
			win.removeEventListener("mouseover",advancedMSBuilder.mouseOver,true);
			win.removeEventListener("mouseout",advancedMSBuilder.mouseOut,true);
			win.removeEventListener("mousedown",advancedMSBuilder.mouseDown,true);
			win.removeEventListener("mouseup",advancedMSBuilder.mouseUp,true);
			win.removeEventListener("DOMMouseScroll",advancedMSBuilder.mouseScroll,true);
			win.removeEventListener("click",advancedMSBuilder.mouseClick,true);
			win.removeEventListener("keypress",advancedMSBuilder.keyPress,true);
		};
		iterateWindows(content.window, unhookup)
		
		var tb = document.getElementById("content");
		tb.removeEventListener("draggesture",this.handler.dragGesture,true);
	},

	reset: function(){
		var editor = document.getElementById("advanedMSEditor");
		editor.editor.selection
			.QueryInterface(Components.interfaces.nsISelectionPrivate)
	 		.addSelectionListener(msSelectionListener);
		editor.userDocument = content.document;
		editor.callbackFunc = this.valueCallback;

		document.getElementById("bookmark").style.display = "block";
		document.getElementById("save").style.display = "none";
		document.getElementById("title").value = advanceOptions.strings.getString("normalTitle");

		editor.content.body.innerHTML = "";
		this.uuid = null;
		this.attachEventHandlers();
		advanceOptions.reset();
		//Something needed for property
		advanceOptions.broadcaster.setAttribute("disabled",true);
		this.loadExistGenerator();
	},

	valueCallback: function(ele){
		advanceOptions.span = ele;
		if (advanceOptions.isOpen() == true){
			advanceOptions.populate();
			return;
		}

		if (ele){
			advanceOptions.broadcaster.setAttribute("disabled",false);
		}else{
			advanceOptions.broadcaster.setAttribute("disabled",true);
		}
	},
	
	clear: function(){
		var editor = document.getElementById("advanedMSEditor");
		editor.clear();		
	},
	
	toXMLString: function(str) {
		return str.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/\'/g, "&apos;").replace(/\"/g, "&quot;");
	},
	
	toRegexString: function(str) {
		return str.replace(/(\.|\?|\$|\^|\+|\*|\\|\||\(|\{|\[|\)|\}|\])/g, '\\$1');
	},

	generateXSLTString: function(node){
		var str = "";
		switch (node.getAttribute("type")){
			case "newvalue":
				var exp = node.getAttribute("expression");
				var content = node.getAttribute("newvalue");
				if (exp==null)
					return null;
				str += '			<value-of amg:type="newvalue" amg:exp="' + exp + '" select="' + content + '"/>\n';
				return str;
			case "siblingnumber":
				var exp = node.getAttribute("siblingnumber");
				if (exp==null)
					return null;
				str += '			<value-of amg:type="siblingnumber" amg:exp="' + node.getAttribute("expression") + '" select="' + exp + '"/>\n';
				return str;
			case "childrennumber":
				var exp = node.getAttribute("childrennumber");
				if (exp==null)
					return null;
				str += '			<value-of amg:type="childrennumber" amg:exp="' + node.getAttribute("expression") + '" select="' + exp + '"/>\n';
				return str;
			case "predicator":
				var exp = node.getAttribute("predicatortype");
				if (exp==null)
					return null;
				var condition = node.getAttribute("condition");
				var ptype = node.getAttribute("predicatortype");
				str += '			<choose amg:type="predicator"' + ' amg:predicator="' + ptype + '" amg:exp="' + node.getAttribute("expression") + '">\n';
				var ifexp = node.getAttribute(ptype + "-if");
				var elseexp = node.getAttribute(ptype + "-else");
				
				switch (ptype){
					case "valuechanged":
					case "siblingchanged":
					case "childrenchanged":
						if (ifexp){
							str += '				<when test="' + condition + '">\n';
							var name = ptype + "-if-" + ifexp;
							var value = node.getAttribute(name);
							switch (ifexp){
								case "selectFromPage":
								case "numberOfNewSiblings":
								case "numberOfNewChildren":
									if (value){
										str += '					<value-of ' + 'amg:actiontype="' + ifexp + '" select="' + value + '"/>\n';
										str += '				</when>\n';
									}
	  								break;
								case "defineBySelf":
									if (value){
										str += '					<text>' + value + '</text>\n';
										str += '				</when>\n';
									}	
									break;
								default:
									str += '				</when>\n';																		
							}
						}
						if (elseexp){
							str += '				<otherwise>\n';
							var name = ptype + "-else-" + elseexp;
							var value = node.getAttribute(name);
							switch (elseexp){
								case "selectFromPage":
								case "numberOfNewSiblings":
								case "numberOfNewChildren":
									if (value){
										str += '					<value-of ' + 'amg:actiontype="' + elseexp + '" select="' + value + '"/>\n';
										str += '				</otherwise>\n';
									}
									break;
								case "defineBySelf":
									if (value){
										str += '					<text>' + value + '</text>\n';
										str += '				</otherwise>\n';
									}
									break;
								default:
									str += '				</otherwise>\n';
							}
						}
						break;
						
					default:
						return null;
				}
				str += '			</choose>\n';
				break;
			default:
				return null;
		}
		return str;
	},

	buildTemplateString: function() {
		var title = this.mseditor.userDocument.title;
		if (!this.uuid)
			this.uuid = this.ug.generateUUID();
		var str = '<?xml version="1.0" encoding="UTF-8"?>\n'
		+ '<generator xmlns="http://www.mozilla.org/microsummaries/0.1" xmlns:amg="http://advmicrosummarygenerator.mozdev.org/custom" name="Microsummary for '
		+ this.toXMLString(title) + '" '
		+ 'uri="urn:' + this.uuid  + '">\n'
		+ ' <template>\n'
		+ '   <transform xmlns="http://www.w3.org/1999/XSL/Transform" version="1.0">\n'
		+ '     <output method="text"/>\n'
		+ '     <template match="/">\n';
		
		var nodes = this.mseditor.content.body.childNodes;
		for (var i = 0 ; i < nodes.length; i ++){
			var node = nodes[i];
			switch (node.nodeType) {
      				case this.mseditor.content.TEXT_NODE:
        				str += '			<text>' + this.toXMLString(node.nodeValue) + '</text>\n';
          				break;
        			case this.mseditor.content.ELEMENT_NODE:
        				if (node.localName != "SPAN" || !node.hasAttribute("expression")){
        					dump("Something wrong in MSEditor\n");
        					break;
        				}
					var xslt = this.generateXSLTString(node);
					if (xslt != null)
						str += xslt;
          				break;
      			}
		}
		
		str = str
		+ '     </template>\n'
		+ '   </transform>\n'    
		+ ' </template>\n'
		+ ' <pages>\n'
		+ '   <include>^';
		
		str += this.toXMLString( this.toRegexString(this.mseditor.userDocument.location.toString())) + '$</include>\n';
		
		str = str
		+ '  </pages>\n'
		+ '</generator>\n';
		
		return str;
	},

	loadExistGenerator: function() {
		var ios = Cc["@mozilla.org/network/io-service;1"]
                    			.getService(Ci.nsIIOService);
		var uri = ios.newURI(content.document.location,null,null);
		var ms = this.mss.getMicrosummaries(uri , -1);
		var it = ms.QueryInterface(Ci.nsIMicrosummarySet).Enumerate();
		while (it.hasMoreElements()){
			var m = it.getNext();
			m = m.QueryInterface(Ci.nsIMicrosummary);
			var url = m.generator.uri.spec;
			if (url.search(/urn:\{[0-9a-fA-F-]+\}/) == -1)
				continue;

			if (this.bookmarked( uri, m )){
				document.getElementById("bookmark").style.display = "none";
				document.getElementById("save").style.display = "block";
			}
			document.getElementById("title").value = advanceOptions.strings.getString("editTitle");
			this.uuid = url.match(/urn:(\{[0-9a-fA-F-]+\})/)[1];
			var fileUrl = m.generator.localURI;
			var fileHandler = ios.getProtocolHandler("file")
								.QueryInterface(Ci.nsIFileProtocolHandler);
			var file = fileHandler.getFileFromURLSpec(fileUrl.spec);
			file = file.QueryInterface(Ci.nsIFile);
			var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                        				.createInstance(Components.interfaces.nsIFileInputStream);
			istream.init(file, 0x01, 0444, 0);
			istream.QueryInterface(Components.interfaces.nsILineInputStream);
			var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
                   						.createInstance(Components.interfaces.nsIConverterInputStream);
			const replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
			is.init(istream, "UTF-8", 1024, replacementChar);
			var lines = "";
			var str = {};
			while (is.readString(4096, str) != 0) {
  				lines += str.value;
			}

			is.close();
			istream.close();
			this.generatorstring = lines;
			break;
		}
		var editor = document.getElementById("advanedMSEditor");
		editor.focus();
	},

	bookmarked: function( uri, microsummary ){
		var bookmarks = this.bms.getBookmarkIdsForURI(uri, {}, {});
		for (var i=0; i<bookmarks.length; i++){
			var bookmark = bookmarks[i];
			if (this.mss.isMicrosummary(bookmark, microsummary))
				return true;
		}
		return false;
	},

	saveMSGenerator: function() {
		//Firstly , we close the editor
		this.closeMSEditor();
	
		//I should say there is some bugs needed to be fixed before this remvoe function can work
		var str = this.buildTemplateString();
		//alert(str);
		var dom = this.dp.parseFromString( str, "text/xml" );
		var generator = this.mss.installGenerator( dom );
		this.refresh(generator);
	},

	selectLiveTitle: function() {
		var menulist = document.getElementById("editBMPanel_namePicker");

		for (var i = 0; i < menulist.childNodes.length; i ++)
			if (menulist.childNodes[i].localName=="menupopup")
				var len = menulist.childNodes[i].childNodes.length;
		menulist.selectedIndex = len - 1;
	},

	deleteListener: function() {
		var win = document.getElementById("editBookmarkPanel");
		win.removeEventListener("popupshown", advancedMSBuilder.selectLiveTitle, true);
		win.removeEventListener("popuphidden", advancedMSBuilder.deleteListener, true);
	},
	
	bookmarkMSGenerator: function() {
		//Firstly , we close the editor
		this.closeMSEditor();
	
		//I should say there is some bugs needed to be fixed before this remvoe function can work
		var str = this.buildTemplateString();
		//alert(str);
		var dom = this.dp.parseFromString( str, "text/xml" );
		this.mss.installGenerator( dom );

		//Change to this one in the trunk code.
		var win = document.getElementById("editBookmarkPanel");
		win.addEventListener("popupshown", advancedMSBuilder.selectLiveTitle, true);		
		win.addEventListener("popuphidden", advancedMSBuilder.deleteListener, true);
		PlacesCommandHook.bookmarkCurrentPage(true, PlacesUtils.bookmarksRootId);
		//win.dispatchEvent("");
	},

	refreshAll: function(){
		var bookmarks = this.ans.getItemsWithAnnotation(FIELD_MICSUM_GEN_URI, {});
		for (var i=0; i < bookmarks.length; i++){
			var bookmark = bookmarks[i];
			this.mss.refreshMicrosummary( bookmark );
		}
	},

	refresh: function( generator ){
		var bookmarks = this.ans.getItemsWithAnnotation(FIELD_MICSUM_GEN_URI, {});
		for (var i=0; i < bookmarks.length; i++){
			var bookmark = bookmarks[i];
			if (this.mss.getMicrosummary(bookmark).generator.equals(generator)){
				this.mss.refreshMicrosummary( bookmark );
			}
		}		
	},

	frameOpened: false,
	changeFrame: function(element){
		if (element.ownerDocument!=content.document&&!this.frameOpened){
			this.frameOpened = true;
			var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
			var result = prompts.confirm(window, advanceOptions.strings.getString("changeFrameTitle"), advanceOptions.strings.getString("changeFrameContent"));
			if (result)
				content.document.location = element.ownerDocument.location;
			this.frameOpened = false;
			return true;
		}
		return false;
	},
	
	handler : {
		context: {
			style: null,
			element: null
		},
		
		mouseDown: function(evt){
			if (evt.button != 0)
				return;

			if (evt.target.tagName == "HTML")
				return;
			var sel = content.window.getSelection();
			if( sel.rangeCount > 0 )
				sel.removeAllRanges();
			var range = content.document.createRange();
			range.selectNode(this.context.element);
			sel.addRange(range);
			
			var elt = evt.target;
			elt.style.outline = this.context.style;
			this.context.style = null;
		},
		
		mouseUp: function(evt){
			if (evt.button != 0)
				return;
		
			if (evt.target.tagName == "HTML")
				return;

			var sel = content.window.getSelection();
			sel.removeAllRanges();
			msStyleOptions.handleMouseUp(this);
		},
		
		mouseOver: function(evt){
			var elt = evt.target;
			this.context.element = elt;
			this.context.style = elt.style.outline;
			//Make the new style;
			elt.style.outlineWidth = '2px';
			elt.style.outlineStyle = 'solid';
			elt.style.outlineColor = msStyleOptions.outline;
			
		},

		//I found this function will do something wrong with the node inside a form
		mouseOut: function(evt){
			var elt = evt.target;
			//I hope this may not cause definite loop
			if (elt != this.context.element&&this.context.element!=null){
				var ele = this.context.element;
				var outevt = content.document.createEvent("MouseEvents");
				outevt.initMouseEvent("mouseout", true, true, content.window,	
								0, 0, 0, 0, 0, false, false, false, false, 0, null);
				ele.dispatchEvent( outevt );
			}
			elt.style.outline = this.context.style;
			this.context.element = null;
		},
		
		mouseScroll: function(evt) {
			var levels = evt.detail;

			if (levels < 0){
				var times = levels/-3;
				this.goUp( times );
			}
			else {
				var times = levels/3;
				this.goDown( times );
			}
			
		},

              mouseClick: function(evt) {
              	if (evt.button != 0)
				return;

                     evt.preventDefault();
			evt.stopPropagation();
		},
		
		keyPress: function(evt){
			if (evt.which == evt.DOM_VK_ADD ||
					evt.which == evt.DOM_VK_W){
				this.goUp(1);
			}
			else if (evt.which == evt.DOM_VK_SUBTRACT ||
							 evt.which == evt.DOM_VK_N){
				this.goDown(1);
			}
			
		},
		
		dragGesture: function(evt){
			if (advancedMSBuilder.changeFrame(evt.target)){
				//evt.preventDefault();
				//evt.stopPropragation();
				return;
			}
			
			//Now I add a work around to deal with the drag
			if(evt.target.localName != "HTML"){
				nsDragAndDrop.startDrag(evt, advancedMSBuilder.dragAndDropObserver ); 
			}
		},
		
		//below is some helper functions
		goUp: function( times ){
			for (var i = 0; i < times; i ++){
				if (advancedMSBuilder.handler.context.element != null &&
						advancedMSBuilder.handler.context.element.parentNode != null){
					var origin = advancedMSBuilder.handler.context.element;
					var outevt = content.document.createEvent("MouseEvents");
					outevt.initMouseEvent("mouseout", true, true, content.window,	
									0, 0, 0, 0, 0, false, false, false, false, 0, null);
					origin.dispatchEvent(outevt);
					
					var inevt = content.document.createEvent("MouseEvents");
					inevt.initMouseEvent("mouseover",true,true,content.window,
								0, 0, 0, 0, 0, false, false, false, false, 0, null);
					
					origin.parentNode.dispatchEvent(inevt);
				}
			}
		},
		
		goDown: function( times ){
			for (var i = 0; i < times; i ++)
				if (advancedMSBuilder.handler.context.element != null &&
						advancedMSBuilder.handler.context.element.childNodes != null){
					var origin = advancedMSBuilder.handler.context.element;
					var outevt = content.document.createEvent("MouseEvents");
					outevt.initMouseEvent("mouseout", true, true, content.window,	
									0, 0, 0, 0, 0, false, false, false, false, 0, null);
					origin.dispatchEvent(outevt);
					
					var inevt = content.document.createEvent("MouseEvents");
					inevt.initMouseEvent("mouseover",true,true,content.window,
								0, 0, 0, 0, 0, false, false, false, false, 0, null);
					
					var nodes = origin.childNodes;
					var node = null;
					for ( var i = 0 ; i < nodes.length; i ++){
						if ( nodes[i].nodeType == 1){
							node = nodes[i];
							break;
						}
					}
					if ( node != null)
						node.dispatchEvent(inevt);
				}
		}
	},
	
	dragAndDropObserver : {
		onDragStart: function( aEvent, aXferData, aDragAction)
		{
			var expression = "string(" + getXPath(advancedMSBuilder.handler.context.element) + ")";
    			aXferData.data = new TransferData();
    			aXferData.data.addDataForFlavour("text/unicode", expression);
		}
	}

};

/*
   An advanced node is some selected node which does not use the node value as the bookmark value. 
   An advanced node has the following attributes:

   type: 
   the node type. newvalue for the node value/siblingnumber for the number of its siblings/predicator indicate that the node is using predicator

   siblingnumber:
   the xpath expression for get the number of siblings.

   predicatortype: 
   the type of predicator. Now we only support two types of predicators. One is valuechanged and the other is siblingchanged

   valuechanged-if:
   text value definition of value changed predicator for if condition.

   valuechanged-else:
   text value definition of value chagned predicator for else condition.

   siblingchanged-if:
   text value definition of sibling chagned predicator for if condition.

   siblingchanged-else:
   text value definition of sibling changed predicator for else condition.
*/

var advanceOptions = {
	span: null,

	//The whole advance element
	_element: null,
	get element(){
		if (this._element == null)
			this._element = document.getElementById("advanced");
		return this._element;
	},

	//The string bundle used for I18N
	_strings: null,
	get strings() {
		if (this._strings == null)
			this._strings = document.getElementById("strs");
		return this._strings;
	},

	//The button which control the display/hide of advance options
	_button: null,
	get button() {
		if (this._button == null)
			this._button = document.getElementById("advanceButton");
		return this._button;
	},

	//The broadcaster which used to change the value of the button
	_broadcaster: null,
	get broadcaster() {
		if (this._broadcaster == null)
			this._broadcaster = document.getElementById("toogleAdvanced");
		return this._broadcaster;
	},

	_monitorelement: null,
	get monitorelement() {
		if (this._monitorelement == null)
			this._monitorelement = document.getElementById("monitor");
		return this._monitorelement;
	},

	//The if clousure hbox
	_if: null,
	get ifelement() {
		if (this._if == null)
			this._if = document.getElementById("if");
		return this._if;
	},

	//The menulist element
	_ifvalue: null,
	get ifmenulist() {
		if (this._ifvalue == null)
			this._ifvalue = document.getElementById("if-option");
		return this._ifvalue;
	},

	_iftext: null,
	get iftext() {
		if (this._iftext == null)
			this._iftext = document.getElementById("if-text");
		return this._iftext;
	},

	//The else clousure hbox
	_else: null,
	get elseelement() {
		if (this._else == null)
			this._else = document.getElementById("else");
		return this._else;
	},

	//The else menulist element
	_elsevalue: null,
	get elsemenulist() {
		if (this._elsevalue == null)
			this._elsevalue = document.getElementById("else-option");
		return this._elsevalue;
	},

	_elsetext: null,
	get elsetext() {
		if (this._elsetext == null)
			this._elsetext = document.getElementById("else-text");
		return this._elsetext;
	},

	//The menulist for selecting node types
	_types: null,
	get types() {
		if (this._types == null)
			this._types = document.getElementById("types");
		return this._types;
	},

	//The menulist for selecting predicator types
	_predicator: null,
	get predicatortypes() {
		if (this._predicator == null)
			this._predicator = document.getElementById("predicator");
		return this._predicator;
	},

	_xmleditor: null,
	get xmleditor() {
		if (this._xmleditor==null)
			this._xmleditor = document.getElementById("xml");
		return this._xmleditor;
	},

	_xmlbutton: null,
	get xmlbutton() {
		if (this._xmlbutton==null)
			this._xmlbutton = document.getElementById("xmlbutton");
		return this._xmlbutton;
	},

	_caption:null,
	get propertyCaption() {
		if (this._caption==null)
			this._caption = document.getElementById("property");
		return this._caption;
	},

	_editbox: null,
	get editbox() {
		if (this._editbox==null)
			this._editbox = document.getElementById("editbox");
		return this._editbox;
	},
	
	empty: function( str, label, input, event ) {
		label.value = "";
		input.value = "";
		label.style.display = "none";
		input.style.display = "none";
		msStyleOptions.reset();
	},

	// the argument which is used to indicate whether the change come from if clousure or else
	selectFromPage: function( str, label, input, event ) {
		input.style.display = "none";
		label.style.display = "block";

		var name = str + "-selectFromPage";
		msStyleOptions.outline = "red";
		msStyleOptions.common = false;
		msStyleOptions.name = name;
		msStyleOptions.node = this.span;
		msStyleOptions.label = label;
	},

	numberOfNewSiblings: function( str, label, input, event ) {
		var name = str + "-numberOfNewSiblings";
		input.style.display = "none";
		label.style.display = "block";

		var exp = this.siblingExp( this.span.getAttribute("expression"));
		var ret = this.evaluate(exp);
		if (ret == null){
			dump("Something error in calculate the number of node sibling, the xpath is : " + exp);
			return;
		}

		exp = exp + "-" + ret;
		this.span.setAttribute(name,exp);
		this.refreshNode();

		label.value = this.evaluate(exp);
		msStyleOptions.reset();
	},

	numberOfNewChildren: function( str, label, input, event) {
		var name = str + "-numberOfNewChildren";
		input.style.display = "none";
		label.style.display = "block";

		var exp = this.siblingExp( this.span.getAttribute("expression"));
		var ret = this.evaluate(exp);
		if (ret == null){
			dump("Something error in calculate the number of node sibling, the xpath is : " + exp);
			return;
		}

		exp = exp + "-" + ret;
		this.span.setAttribute(name,exp);
		this.refreshNode();

		label.value = this.evaluate(exp);
		msStyleOptions.reset();
	},

	defineBySelf: function( str, label, input, event ) {
		var name = str + "-defineBySelf";
		//To remove the last value, here
		if (this.span.getAttribute(name)==null)
			input.value = "";
		input.style.display = "block";
		label.style.display = "none";
		msStyleOptions.reset();
	},

	selectFromPageItem: function( ele ) {
		ele.appendItem(this.strings.getString("selectFromPage"),"selectFromPage");
	},

	numberOfNewSiblingItem: function( ele ) {
		ele.appendItem(this.strings.getString("nubmerOfNewSiblings"),"numberOfNewSiblings");
	},

	numberOfNewChildrenItem: function( ele ) {
		ele.appendItem(this.strings.getString("numberOfNewChildren"),"numberOfNewChildren");
	},
		
	defineBySelfItem: function( ele ) {
		ele.appendItem(this.strings.getString("defineBySelf"),"defineBySelf");
	},

	nullItem: function( ele ) {
		ele.appendItem("                  ","empty");
	},

	isOpen: function (){
		return this.element.collapsed == false;
	},
	
	//Show the advance options
	open: function(event){
		if (this.span==null){
			return;
		}

		this.element.collapsed = false;
		this.populate();
		//this.button.setAttribute("command", "closeAdvance");
		//this.button.setAttribute("observes", "closeAdvanced");
		//this.button.command = document.getElemenetById("closeAdvance");
		//this.broadcaster.setAttribute("oncommand","advanceOptions.close()");
		//this.broadcaster.setAttribute("label",this.strings.getString("hideAdvance"));
		this.broadcaster.setAttribute("disabled",true);

		//we should hide the xml editing box
		this.xmleditor.collapsed = true;
	},

	//Hide the advance options
	close: function(){
		this.element.collapsed = true;
		this.broadcaster.setAttribute("disabled",false);
		//this.button.setAttribute("command", "openAdvance");
		//this.button.setAttribute("observes", "openAdvanced");
		//this.broadcaster.setAttribute("oncommand","advanceOptions.open(event)");
		//this.broadcaster.setAttribute("label",this.strings.getString("showAdvance"));

		//To see whether the xml editing box is open
		if (this.xmlbutton.getAttribute("disabled")==Boolean(true)){
			this.xmleditor.collapsed = false;
		}
	},

	reset: function() {
		this.close();
		this.cancelEdit();
		this.span = null;
	},

	editXML: function() {
		this.xmleditor.collapsed = false;
		this.close();
		this.editbox.value =  advancedMSBuilder.generatorstring;
		this.xmlbutton.setAttribute("disabled", true);
	},

	cancelEdit: function() {
		this.xmleditor.collapsed = true;
		this.xmlbutton.setAttribute("disabled", false);

		document.getElementById("advanedMSEditor").focus()
	},

	saveEdit: function() {
		this.xmleditor.collapsed = true;
		var e = document.getElementById("advanedMSEditor");
		e.focus();

		this.xmlbutton.setAttribute("disabled", false);
		advancedMSBuilder.generatorstring = this.editbox.value;
	},

	//Change the SPAN node value after the node's attribute changed
	refreshNode: function() {
		switch (this.span.getAttribute("type")) {
			case "newvalue": 
				var exp = this.span.getAttribute("newvalue");
				var ret = this.evaluate(exp) 
				if (ret != null)
					this.span.textContent = ret;
				break;
			case "siblingnumber":
				var exp = this.span.getAttribute("siblingnumber");
				var ret = this.evaluate(exp);
				if (ret != null)
					this.span.textContent = ret;
				break;
			case "childrennumber":
				var exp = this.span.getAttribute("childrennumber");
				var ret = this.evaluate(exp);
				if (ret!=null)
					this.span.textContent = ret;
				break;
			case "predicator":
				//Here we need to process more
				var exp = this.span.getAttribute("condition");
				var ret = this.evaluate(exp);
				if (ret){
					exp = this.span.getAttribute("predicatortype") + "-if";
				}else{
					exp = this.span.getAttribute("predicatortype") + "-else";
				}
				switch (this.span.getAttribute(exp)){
					case "selectFromPage":
					case "numberOfNewSiblings":
						var valueExp = this.span.getAttribute(exp+"-"+this.span.getAttribute(exp));
						ret = this.evaluate(valueExp);
						if (ret!=null)
							this.span.textContent = ret;
						break;
					case "defineBySelf":
						var value = this.span.getAttribute(exp+"-"+this.span.getAttribute(exp));
						this.span.textContent = value;
						break;
					default:
						this.span.textContent = "... ...";
				}
				break;
			default:
				dump("There must be undefined type here!");			
		}
	},

	//Below are methods used to manipulate the node types
	populate: function(  ) {
		if (this.span==null){
			this.close();
			this.broadcaster.setAttribute("disabled",true);
			return;
		}

		var ele = document.getElementById("types-popup");
		this.buildTypeLabel(ele);
		this.buildCaption();
		
		//this.monitorelement.style.display = "none";
		//this.ifelement.style.display = "none";
		//this.elseelement.style.display = "none";
		this.predicatortypes.setAttribute("disabled",true);
		this.ifmenulist.setAttribute("disabled",true);
		this.elsemenulist.setAttribute("disabled",true);
		document.getElementById("if-label").style.display = "none";
		document.getElementById("if-input").style.display = "none";
		document.getElementById("else-label").style.display = "none";
		document.getElementById("else-input").style.display = "none";

		switch (this.span.getAttribute("type")) {
			case "newvalue": 
				this.types.selectedIndex = 0;
				break;
			case "siblingnumber":
				this.types.selectedIndex = 1;
				break;
			case "childrennumber":
				this.types.selectedIndex = 2;
				break;
			case "predicator":	
				this.element.setAttribute("disabled",false);
				this.types.selectedIndex = 3;
				this.populatePredicator();
				break;
			default:
				dump("There must be undefined type here!");
		}
	},

	//Change the bookmark value to the value of the node
	newvalue: function() { 
		//this.monitorelement.style.display = "none";
		//this.ifelement.style.display = "none";
		//this.elseelement.style.display = "none";
		this.predicatortypes.setAttribute("disabled",true);
		this.ifmenulist.setAttribute("disabled",true);
		this.elsemenulist.setAttribute("disabled",true);

		document.getElementById("if-label").style.display = "none";
		document.getElementById("if-input").style.display = "none";
		document.getElementById("else-label").style.display = "none";
		document.getElementById("else-input").style.display = "none";


		this.span.setAttribute("type","newvalue");
		this.span.setAttribute("newvalue", "normalize-space(" + this.span.getAttribute("expression") + ")");
		this.refreshNode();
	},

	//Change the bookmark value to the number of the selected node siblings
	siblingnumber: function() {
		//this.monitorelement.style.display = "none";
		//this.ifelement.style.display = "none";
		//this.elseelement.style.display = "none";
		this.predicatortypes.setAttribute("disabled",true);
		this.ifmenulist.setAttribute("disabled",true);
		this.elsemenulist.setAttribute("disabled",true);

		document.getElementById("if-label").style.display = "none";
		document.getElementById("if-input").style.display = "none";
		document.getElementById("else-label").style.display = "none";
		document.getElementById("else-input").style.display = "none";


		this.span.setAttribute("type","siblingnumber");
		var arr = this.span.getAttribute("expression");
		var str = this.siblingExp( arr );
		
		this.span.setAttribute("siblingnumber",str);

		this.refreshNode();
	},

	//Change the bookmark value to the number of the selected node children
	childrennumber: function() {
		//this.monitorelement.style.display = "none";
		//this.ifelement.style.display = "none";
		//this.elseelement.style.display = "none";
		this.predicatortypes.setAttribute("disabled",true);
		this.ifmenulist.setAttribute("disabled",true);
		this.elsemenulist.setAttribute("disabled",true);

		document.getElementById("if-label").style.display = "none";
		document.getElementById("if-input").style.display = "none";
		document.getElementById("else-label").style.display = "none";
		document.getElementById("else-input").style.display = "none";


		this.span.setAttribute("type","childrennumber");
		var arr = this.span.getAttribute("expression");
		var str = this.childrenExp( arr );
		
		this.span.setAttribute("childrennumber",str);

		this.refreshNode();
	},

	//Change the node type to use predicator
	predicator: function() {
		this.monitorelement.setAttribute("disabled",false);
		this.span.setAttribute("type","predicator");

		this.populatePredicator();
	},

	//method used to process node type changing. (New value/Sibling number/Predicator)
	handleCommand: function( event ) {
		//this.types[0](); == this.types.0() javascript's array is very intersting
		if (this.span == null) {
			this.types.selectedIndex = 0;
			return;
		}

		this.span.setAttribute("type",event.target.value);
		this.refreshNode();
		this[event.target.value]();
	},

	//Below is methods used by predicators 
	populatePredicator: function() {
		//this.predicatortypes.style.display = "block";
		//this.ifelement.style.display = "block";
		//this.elseelement.style.display = "block";
		this.predicatortypes.setAttribute("disabled",false);
		this.ifmenulist.setAttribute("disabled",false);
		this.elsemenulist.setAttribute("disabled",false);
		

		switch (this.span.getAttribute("predicatortype")) {
			case "valuechanged":
				this.predicatortypes.selectedIndex = 1;
				this.ifmenulist.removeAllItems();
				var nodeValue = this.strings.getString("nodeValue");
				this.iftext.value = this.strings.getString("whenTrue");
				this.nullItem(this.ifmenulist);
				this.defineBySelfItem(this.ifmenulist);
				this.selectFromPageItem(this.ifmenulist);

				this.elsemenulist.removeAllItems();
				this.elsetext.value = this.strings.getString("whenFalse");
				this.nullItem(this.elsemenulist);
				this.defineBySelfItem(this.elsemenulist);
				this.selectFromPageItem(this.elsemenulist);

				this.populateValue("valuechanged-if");
				this.populateValue("valuechanged-else");
				break;
			case "siblingchanged":
				this.predicatortypes.selectedIndex = 2;
				this.ifmenulist.removeAllItems();
				var siblingNumber = this.strings.getString("siblingNumber");
				this.iftext.value = this.strings.getString("whenTrue");
				this.nullItem(this.ifmenulist);
				this.defineBySelfItem(this.ifmenulist);
				this.selectFromPageItem(this.ifmenulist);
				this.numberOfNewSiblingItem(this.ifmenulist);

				this.elsemenulist.removeAllItems();
				this.elsetext.value = this.strings.getString("whenFalse");
				this.nullItem(this.elsemenulist);
				this.defineBySelfItem(this.elsemenulist);
				this.selectFromPageItem(this.elsemenulist);

				this.populateValue("siblingchanged-if");
				this.populateValue("siblingchanged-else");
				break;
			case "childrenchanged":
				this.predicatortypes.selectedIndex = 3;
				this.ifmenulist.removeAllItems();
				var childrenNumber = this.strings.getString("childrenNumber");
				this.iftext.value = this.strings.getString("whenTrue");
				this.nullItem(this.ifmenulist);
				this.defineBySelfItem(this.ifmenulist);
				this.selectFromPageItem(this.ifmenulist);
				this.numberOfNewChildrenItem(this.ifmenulist);

				this.elsemenulist.removeAllItems();
				this.elsetext.value = this.strings.getString("whenFalse");
				this.nullItem(this.elsemenulist);
				this.defineBySelfItem(this.elsemenulist);
				this.selectFromPageItem(this.elsemenulist);

				this.populateValue("childrenchanged-if");
				this.populateValue("childrenchanged-else");
				break;
			default:
				this.predicatortypes.selectedIndex = 0;
				this.populateValue("if");
				this.populateValue("");
				//this.ifelement.style.display = "none";
				//this.elseelement.style.display = "none";
				this.ifmenulist.setAttribute("disabled",true);
				this.elsemenulist.setAttribute("disabled",true);				

		}
	},

	//There are four fields for predicator, they are 
	//valuechanged-if/valuechanged-else/siblingchanged-if/siblingchanged-else
	//each of these fields can be one of selectFromPage/numberOfNewSiblings/defineBySelf
	//And for each value of above combination, their attribute names are:
	//valuechanged-if-selecFromPage
	//valuechanged-if-numberOfNewSiblings
	//valuechanged-if-defineBySelf
	//... ...
	populateValue: function( type ) {
		var label, input, menulist;
		if (type.match(/if/)){
			label = document.getElementById("if-label");
			input = document.getElementById("if-input");
			menulist = this.ifmenulist;
		}else {
			label = document.getElementById("else-label");
			input = document.getElementById("else-input");			
			menulist = this.elsemenulist;
		}
		
		switch (this.span.getAttribute(type)){
			case "selectFromPage":
				var exp = this.span.getAttribute(type + "-selectFromPage");
				var ret = this.evaluate(exp);
				if (ret != null){
					input.style.display = "none";
					label.style.display = "block";
					label.value = '" ' + ret + ' "';
				}
				menulist.selectedIndex = 2;
				break;
			case "numberOfNewSiblings":
				var exp = this.span.getAttribute(type + "-numberOfNewSiblings");
				var ret = this.evaluate(exp);
				if (ret != null){
					input.style.display = "none";
					label.style.display = "block";
					label.value = '" ' + ret + ' "';					
				}

				menulist.selectedIndex = 3;
				break;
			case "numberOfNewChildren":
				var exp = this.span.getAttribute(type + "-numberOfNewChildren");
				var ret = this.evaluate(exp);
				if (ret != null){
					input.style.display = "none";
					label.style.display = "block";
					label.value = '" ' + ret + ' "';					
				}

				menulist.selectedIndex = 3;
				break;
			case "defineBySelf":
				var ret = this.span.getAttribute(type + "-defineBySelf");
				input.style.display = "block";
				label.style.display = "none";
				input.value = ret;

				menulist.selectedIndex = 1;
				break;
			default:
				input.style.display = "none";
				label.style.display = "none";
				menulist.selectedIndex = 0;
		}
	},

	emptyType: function() {
		//this.ifelement.style.display = "none";
		//this.elseelement.style.display = "none";
		this.ifmenulist.setAttribute("disabled",true);
		this.elsemenulist.setAttribute("disabled",true);
	},

	valueChanged: function() {
		this.span.setAttribute("predicatortype","valuechanged");
		var exp = this.span.getAttribute("newvalue");
		var ret = this.evaluate(exp);
		exp = exp + "!='" + ret + "'";
		this.span.setAttribute("condition",exp);

		this.ifmenulist.removeAllItems();
		var nodeValue = this.strings.getString("nodeValue");
		this.iftext.value = this.strings.getString("whenTrue");
		this.nullItem(this.ifmenulist);
		this.defineBySelfItem(this.ifmenulist);
		this.selectFromPageItem(this.ifmenulist);

		this.elsemenulist.removeAllItems();
		this.elsetext.value = this.strings.getString("whenFalse");
		this.nullItem(this.elsemenulist);
		this.defineBySelfItem(this.elsemenulist);
		this.selectFromPageItem(this.elsemenulist);

		//this.ifelement.style.display = "block";
		//this.elseelement.style.display = "block";
		this.ifmenulist.setAttribute("disabled",false);
		this.elsemenulist.setAttribute("disabled",false);

		this.populatePredicator();
	},

	siblingChanged: function() {
		this.span.setAttribute("predicatortype","siblingchanged");
		var exp = this.siblingExp(this.span.getAttribute("expression"));
		var ret = this.evaluate(exp);
		exp = exp + "!=" + ret;
		
		this.span.setAttribute("condition",exp);
	
		this.ifmenulist.removeAllItems();
		var siblingNumber = this.strings.getString("siblingNumber");
		this.iftext.value = this.strings.getString("whenTrue");
		this.nullItem(this.ifmenulist);
		this.defineBySelfItem(this.ifmenulist);
		this.selectFromPageItem(this.ifmenulist);
		this.numberOfNewSiblingItem(this.ifmenulist);

		this.elsemenulist.removeAllItems();
		this.elsetext.value = this.strings.getString("whenFalse");
		this.nullItem(this.elsemenulist);
		this.defineBySelfItem(this.elsemenulist);
		this.selectFromPageItem(this.elsemenulist);

		//this.ifelement.style.display = "block";
		//this.elseelement.style.display = "block";
		this.ifmenulist.setAttribute("disabled",false);
		this.elsemenulist.setAttribute("disabled",false);
		this.populatePredicator();
	},

	childrenChanged: function() {
		this.span.setAttribute("predicatortype","childrenchanged");
		var exp = this.childrenExp(this.span.getAttribute("expression"));
		var ret = this.evaluate(exp);
		exp = exp + "!=" + ret;
		
		this.span.setAttribute("condition",exp);
	
		this.ifmenulist.removeAllItems();
		var childrenNumber = this.strings.getString("childrenNumber");
		this.iftext.value = this.strings.getString("whenTrue");
		this.nullItem(this.ifmenulist);
		this.defineBySelfItem(this.ifmenulist);
		this.selectFromPageItem(this.ifmenulist);
		this.numberOfNewChildrenItem(this.ifmenulist);

		this.elsemenulist.removeAllItems();
		this.elsetext.value = this.strings.getString("whenFalse");
		this.nullItem(this.elsemenulist);
		this.defineBySelfItem(this.elsemenulist);
		this.selectFromPageItem(this.elsemenulist);

		//this.ifelement.style.display = "block";
		//this.elseelement.style.display = "block";
		this.ifmenulist.setAttribute("disabled",false);
		this.elsemenulist.setAttribute("disabled",false);
		this.populatePredicator();		
	},
	
	predicatorHandleCommand: function( event ) {
		this[event.target.value]();
	},

	valueHandleCommand: function( which, event) {
		var label, input, str;
		if (which){
			str = this.span.getAttribute("predicatortype") + "-if";
			label = document.getElementById("if-label");
			input = document.getElementById("if-input");
		}else{
			str = this.span.getAttribute("predicatortype") + "-else";
			label = document.getElementById("else-label");
			input = document.getElementById("else-input");
		}
		this.span.setAttribute(str,event.target.value);
		this[event.target.value](str, label , input ,event);
	},

	evaluate: function( exp ) {
      		var evaluator = Cc["@mozilla.org/dom/xpath-evaluator;1"].createInstance(Ci.nsIDOMXPathEvaluator);
      		var resolver = evaluator.createNSResolver(content.document);
      		var result;
		try{
			result = evaluator.evaluate( exp, content.document, resolver, Ci.nsIDOMXPathResult.ANY_TYPE,null);
		}catch(e){
			return null;
		}
      		var ret ;
		//I return directly from number_type and bool_type because the number 0 and false are both equal to ""
		//That is so intersting!
      		switch( result.resultType ){
      			case Ci.nsIDOMXPathResult.NUMBER_TYPE:
      				return result.numberValue;
      			    	
      			case Ci.nsIDOMXPathResult.STRING_TYPE:
      				ret = result.stringValue;
      			    	break;
			case Ci.nsIDOMXPathResult.UNORDERED_NODE_ITERATOR_TYPE:
      				ret = null;
      			    	break;
			case Ci.nsIDOMXPathResult.BOOLEAN_TYPE:
				return result.booleanValue;
				
			default:
      			    	ret = null;
      		}
      		if( ret == "" )
      			ret = null;
      		return ret;
	},

	siblingExp: function ( arr ){
		//The expression are some format of 'string(xpath)'
		//So match is needed to extract the real xpath
		arr = arr.match(/\((.*)\)/)[1];
		arr = arr.split("/");
		for ( var i = arr.length - 1; i > 0; i -- ){
			//Find the node with format  node[1] which means it has siblings
			if (arr[i].match(/\[/)){
				arr[i] = arr[i].replace(/\[.*\]/, "");
				break;
			}
		}
		//Compose the new xpath for count the siblings number
		var str = "count(";
		for ( var i = 1; i < arr.length; i ++ )
			str += "/" + arr[i];
		str += ")";

		return str;
	},

	childrenExp: function (arr) {
		arr = arr.match(/\((.*)\)/)[1];
		arr += "/*";
		//Compose the new xpath for count the siblings number
		var str = "count(" + arr + ")";

		return str;
	},

	inputChanged: function( event , indicator ){
		var node = this.span;
		if (node==null)
			return;
		
		if (indicator)
			var ind = "if";
		else
			var ind = "else";

		var ptype = node.getAttribute("predicatortype");
		var name = ptype + "-" + ind ;
		name += "-defineBySelf";
		node.setAttribute(name, event.target.value);

		this.refreshNode();
	},

	showType: function (event, element) {
		this.buildTypeLabel(element);
	},

	buildTypeLabel: function (element){
		var exp = this.span.getAttribute("expression");
		var siblingExp = this.siblingExp(exp);
		var childrenExp = this.childrenExp(exp);

		var ele = element.firstChild;
		ele.label = this.strings.getString("content")  + this.evaluate(exp);
		ele = ele.nextSibling;
		ele.label = this.strings.getString("sibling") + this.evaluate(siblingExp);
		ele = ele.nextSibling;
		ele.label = this.strings.getString("children") + this.evaluate(childrenExp);
	},

	buildCaption: function(){
		var exp = this.span.getAttribute("expression");
		var content = this.evaluate(exp);
		if (content==null)
			content = "";
		if (content.length > 100)
			content = content.slice(0,100) + "... ...";
		var label = this.strings.getFormattedString("property",[exp,content]);
		this.propertyCaption.label = label;
	}
};

//Register for global tab events
window.addEventListener("load",function(){
	var container = gBrowser.tabContainer;
	container.addEventListener("TabClose", tabClosed, false);
	container.addEventListener("TabSelect",tabChanged,false);
	advancedMSBuilder.initPopup();
},false);
