/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Bo Yang<techrazy.yang@gmail.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const FIELD_MICSUM_GEN_URI    = "microsummary/generatorURI";
const FIELD_MICSUM_EXPIRATION = "microsummary/expiration";

var msDragAndDrop = {

  	_mss: null,
  	get mss() {
    		if(!this._mss)
      			this._mss = Cc["@mozilla.org/microsummary/service;1"].
                  		getService(Ci.nsIMicrosummaryService);
    		return this._mss;
  	},

	_dp: null,
	get dp() { 
		if(!this._dp)
			this._dp = Cc["@mozilla.org/xmlextras/domparser;1"].
								 createInstance(Ci.nsIDOMParser);
		return this._dp;
	},
	
	_ug: null,
	get ug() {
		if(!this._ug)
			this._ug = Cc["@mozilla.org/uuid-generator;1"].
								 getService(Ci.nsIUUIDGenerator);	
		return this._ug;
	},

	get _updateInterval() {
    		var updateInterval =
      			getPref("browser.microsummary.updateInterval", 30);
    		// the minimum update interval is 1 minute
    		return Math.max(updateInterval, 1) * 60 * 1000;
  	},

	init: function() {
		document.getElementById("content").addEventListener( "draggesture", this.dragStart , true );		
	},

	onDragStart: function (aEvent, aXferData, aDragAction ){
		var dataSet = new TransferDataSet();
		var data = new TransferData();

		var expression = "string(" + getXPath(aEvent.target) + ")";
		var title = content.document.evaluate( expression, content.document, null, 0, null).stringValue;

		var result = msDragAndDrop.buildTemplateString(title, expression);
		var dom = msDragAndDrop.dp.parseFromString( result[0], "text/xml" );
		msDragAndDrop.mss.installGenerator( dom );

		var now = Date.now();
		var expire = now + msDragAndDrop._updateInterval;

		//Compse the annotations needed here
		var item = title + "\n" + content.document.location.href + "\n" + Components.interfaces.nsIAnnotationService.TYPE_STRING + ":"+ FIELD_MICSUM_GEN_URI + ":" + result[1] + ":0:0\n" + Components.interfaces.nsIAnnotationService.TYPE_INT32 + ":" + FIELD_MICSUM_EXPIRATION + ":" + expire + ":0:0"

		data.addDataForFlavour("text/x-annotated-url", item);
		dataSet.push(data);

		aXferData.data = dataSet ;
		return true;
	},

	dragStart: function(aEvt){
		if(aEvt.target.localName != "HTML"){
			nsDragAndDrop.startDrag(aEvt, msDragAndDrop); 
		}
	},

	toXMLString: function(str) {
		return str.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/\'/g, "&apos;").replace(/\"/g, "&quot;");
	},
	
	toRegexString: function(str) {
		return str.replace(/(\.|\?|\$|\^)/g, '\\$1');
	},

	buildTemplateString: function(title, expression) {
		var uuid = this.ug.generateUUID();
		var str = '<?xml version="1.0" encoding="UTF-8"?>\n'
		+ '<generator xmlns="http://www.mozilla.org/microsummaries/0.1" name="Microsummary for '
		+ this.toXMLString(title) + '" '
		+ 'uri="urn:' + uuid + '">\n'
		+ ' <template>\n'
		+ '   <transform xmlns="http://www.w3.org/1999/XSL/Transform" version="1.0">\n'
		+ '     <output method="text"/>\n'
		+ '     <template match="/">\n';
		
          	str += '<value-of select="' + this.toXMLString(expression) + '"/>\n';	

		str = str
		+ '     </template>\n'
		+ '   </transform>\n'    
		+ ' </template>\n'
		+ ' <pages>\n'
		+ '   <include>';
		
		str += this.toXMLString( content.document.location.toString()) + '</include>\n';
		
		str = str
		+ '  </pages>\n'
		+ '</generator>\n';
		
		return [str,uuid];
	}	
};

function getPref(prefName, defaultValue) {
  	try {
    		var prefBranch = Cc["@mozilla.org/preferences-service;1"].
                     getService(Ci.nsIPrefBranch);
    		switch (prefBranch.getPrefType(prefName)) {
    			case prefBranch.PREF_BOOL:
      				return prefBranch.getBoolPref(prefName);
    			case prefBranch.PREF_INT:
      				return prefBranch.getIntPref(prefName);
    		}
  	}
  	catch (ex) { /* return the default value */ }
  
  	return defaultValue;
}


if (Application.storage.get("draganddrop","no") == "no"){
	Application.storage.set("draganddrop","yes");
	msDragAndDrop.init();
}
