/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Bo Yang<techrazy.yang@gmail.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const MICSUM_NS = new Namespace("http://www.mozilla.org/microsummaries/0.1");

var predicatorUpdater = {
	 _bms: null,
  	get bms() {
    		if (!this._bms)
      			this._bms = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].
                  		getService(Ci.nsINavBookmarksService);
    		return this._bms;
  	},

 	 _ans: null,
  	get ans() {
    		if (!this._ans)
      			this._ans = Cc["@mozilla.org/browser/annotation-service;1"].
                  		getService(Ci.nsIAnnotationService);
    		return this._ans;
  	},

  	_mss: null,
  	get mss() {
    		if (!this._mss)
      			this._mss = Cc["@mozilla.org/microsummary/service;1"].
                  		getService(Ci.nsIMicrosummaryService);
    		return this._mss;
  	},

	_dirs: null,
  	get dirs() {
    		if (!this._dirs)
      			this._dirs = Cc["@mozilla.org/file/directory_service;1"].
                   		getService(Ci.nsIProperties);
    		return this._dirs;
  	},

	_dp: null,
	get dp() { 
		if(!this._dp)
			this._dp = Cc["@mozilla.org/xmlextras/domparser;1"].
								 createInstance(Ci.nsIDOMParser);
		return this._dp;
	},

	init: function(){
		this.bms.addObserver(this, false);
	},

	findXMLFile: function(ms){
		return ms.generator.localURI;
	},
	
	onBeginUpdateBatch: function(){},
	onEndUpdateBatch: function(){},
	onItemAdded: function(aItemId, aFolder, aIndex){},
	onItemRemoved: function(aItemId, aFolder, aIndex){},
	onItemChanged: function(aBookmarkId, aProperty, aIsAnnotationProperty, aValue){},
	onItemMoved: function(aItemId, aOldParent, aOldIndex, aNewParent, aNewIndex){},
	onItemVisited: function(aBookmarkId, aVisitID, time){
		//alert("In Visited");
		if (!predicatorUpdater.mss.hasMicrosummary(aBookmarkId))
			return ;

		var ms = predicatorUpdater.mss.getMicrosummary(aBookmarkId);
		var url = ms.generator.uri.spec;
		if (url.search(/urn:\{[0-9a-fA-F-]+\}/) == -1)
			return;

		var fileUri = predicatorUpdater.findXMLFile(ms);
		//alert(fileUri.spec);
		//var msdir = predicatorUpdater.dirs.get("UsrMicsumGens", Ci.nsIFile);
		var ios = Cc["@mozilla.org/network/io-service;1"]
                    			.getService(Components.interfaces.nsIIOService);
		var fileHandler = ios.getProtocolHandler("file")
                     		.QueryInterface(Ci.nsIFileProtocolHandler);
		var file = fileHandler.getFileFromURLSpec(fileUri.spec);
		file = file.QueryInterface(Ci.nsILocalFile);
		//var file = Cc["@mozilla.org/file/local;1"]
              //      .createInstance(Ci.nsILocalFile);
		//file.initWithPath(filename);
		var ifstream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
		ifstream.init(file, 0x01, 0444, 0);
		ifstream.QueryInterface(Ci.nsILineInputStream);

		var generatorString = "", hasmore = true, line = {};
		do{
			hasmore = ifstream.readLine(line);
			generatorString += line.value + "\n";
		}while(hasmore);
		ifstream.close();

		//alert(generatorString);
		//Change the corresponding predicator part now
		var dom = this.dp.parseFromString(generatorString, "text/xml");
		//I found that getElementsByTagNameNS failed here.
		var predicators = dom.getElementsByTagName("choose");
		var len = 0; 
		for (; len < predicators.length; len ++){
			var when = predicators[len].firstChild;
			if (!when)
				continue;
			//Find the desired node. To skip the text nodes
			while(when && when.nodeType != Ci.nsIDOMNode.ELEMENT_NODE)
				when = when.nextSibling;
			
			var test = when.getAttribute("test");
			var arr = test.split("!=");

			//I need to fetch the dom from here and then evaluate XPath in it, and then update the test string
			//Here I assume the onVisit event is triggered after the page is loaded.
			var result = content.document.evaluate(arr[0],content.document,null,0,null);
			var newvalue = "";
			switch (result.resultType){
				case Ci.nsIDOMXPathResult.NUMBER_TYPE:
					newvalue = result.numberValue;
					break;
				case Ci.nsIDOMXPathResult.STRING_TYPE:
					newvalue = result.stringValue;
					newvalue = "'" + newvalue + "'";
					break;
				default:
					dump("In predicator update service, error!");
					return;
			}

			var newstring = arr[0] + "!=" + newvalue;
			generatorString = generatorString.replace(test, newstring);
			//Change the new siblings's attribute
			generatorString = generatorString.replace(arr[0] + "-" + arr[1], arr[0] + "-" + newvalue,"g");
		}
		
		var ofstream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);		
		ofstream.init(file,0x02 | 0x08 | 0x20, 0666, 0);
		ofstream.write(generatorString, generatorString.length);
		ofstream.close();

		//Refresh the microsummary now
		this.mss.refreshMicrosummary(aBookmarkId);
	}
};

if (Application.storage.get("initialized","no") == "no"){
	Application.storage.set("initialized","yes");
	predicatorUpdater.init();
}
